module Main where

import Data.Text (unpack)

import Tools.PdbqtDevider
import Tools.String
import Tools.IO
import Config


data Fns = Fns {
  progressBar  :: FnProgressBar,
  verbosePutLn :: String -> IO (),
  verbosePut   :: String -> IO ()
  }


extractData :: Fns -> Maybe [String] -> String -> IO ()
extractData fns mbModelNames filename = do
      verbosePutLn fns $ "Process file '" ++ filename ++ "'"
      n <- devidePdbqtModulesFile filename (progressBar fns) getModelFileName
      verbosePutLn fns $ "~> " ++ show n ++ " models created."
      verbosePutLn fns "."
  where
    prefix = (filename `removeFromEnd` ".pdbqt") ++ "_Model_"

    getModelFileName = case mbModelNames of
      Nothing -> \ model -> Just (prefix ++ model ++ ".pdbqt")

      (Just models) -> \ model -> if model `elem` models
        then Just (prefix ++ model ++ ".pdbqt")
        else Nothing


actionListModels :: Fns -> String -> IO ()
actionListModels fns filename = do
  verbosePutLn fns $ "Models in file '" ++ filename ++ "':"
  models <- listModels filename (progressBar fns)
  mapM_ (putStrLn . ("- " ++) . unpack) models
  putStrLn "."


defaultFunctions :: Config -> Fns
defaultFunctions config
  | cfg_quiet config = Fns {
    progressBar  = muteProgressBar,
    verbosePutLn = \ _ -> pure (),
    verbosePut   = \ _ -> pure ()
    }
  | otherwise = Fns {
    progressBar  = textProgressBar,
    verbosePutLn = putStrLn,
    verbosePut   = putStr
    }

main :: IO ()
main = do
  config <- getConfig

  let fns = defaultFunctions config

  case cfg_action config of
    ActionShowHelp      -> showHelp
    ActionShowVersion   -> showVersion
    ActionExtractModels -> mapM_ (extractData fns (cfg_modelSelection config)) (cfg_inputFiles config)
    ActionListModels    -> mapM_ (actionListModels fns) (cfg_inputFiles config)
    a -> putStrLn $ show a ++ " is not supported yet."
