module Tools.IO where

import Data.Text (Text(..))
import Data.Text.IO (hGetLine)
import System.IO (openBinaryFile, IOMode(ReadMode), hClose, hIsEOF, hSeek, hTell, SeekMode(..), hFlush, stdout)

import Tools.String

type FnProgressBar = Maybe Int -> IO ()
type FnLineReader x = x -> Text -> IO x

withJust :: (x -> y) -> y -> Maybe x -> y
withJust j _ (Just x) = j x
withJust _ n Nothing  = n

withLinesInFile :: FilePath -> FnProgressBar -> x -> FnLineReader x -> IO x
withLinesInFile filename progressBar xInit action = do
    h <- openBinaryFile filename ReadMode
    hSeek h SeekFromEnd 0
    fileSize <- hTell h
    hSeek h AbsoluteSeek 0

    let { go lastPos x = do
      p <- hTell h
      let thisPos = fromInteger $ (1000 * p) `div` fileSize
      if thisPos /= lastPos then progressBar (Just thisPos) else pure ()

      eof <- hIsEOF h
      case eof of
        False -> hGetLine h >>= action x >>= go thisPos
        True  -> hClose h >> progressBar Nothing >> pure x
      }

    progressBar (Just 0)
    go 0 xInit


textProgressBar :: FnProgressBar
textProgressBar (Just p) = do
  putStr $ percentText ++ " [" ++ bar ++ "]\r"
  hFlush stdout
  where
    percent = p `div` 10
    percentDot = p `mod` 10
    percentText = paddLeft 6 ' ' $ show percent ++ "." ++ show percentDot ++ "%"
    
    barTotalWidth = 40
    barWidth = (barTotalWidth * p) `div` 1000
    bar = take barTotalWidth $ replicate barWidth '=' ++ repeat ' '

textProgressBar Nothing = putStrLn ""


muteProgressBar :: FnProgressBar
muteProgressBar _ = pure ()
