module Tools.PdbqtDevider where

import Data.Text     (Text (..), pack, unpack, isPrefixOf)
import Data.Text.IO  (hPutStrLn)
import System.IO     (openBinaryFile, IOMode(WriteMode), hClose)

import Tools.String
import Tools.IO

type FnModelToFilename = String -> Maybe String
type FnModelWriter = Maybe Text -> IO ()
type FnModelWriterGenerator x = x -> String -> IO (x, FnModelWriter)


readPdbqtModels :: FilePath -> FnProgressBar -> x -> FnModelWriterGenerator x -> IO x
readPdbqtModels filename progressBar xInit startProcess = withLinesInFile filename progressBar (Nothing, xInit) go >>= finale
    where
        model    = pack "MODEL"
        modelend = pack "ENDMDL"

        finale (Nothing,       x) = pure x
        finale (Just consumer, x) = consumer Nothing >> pure x

        go jc@(Nothing, x) text
            |  model `isPrefixOf` text = do
                (x', consumer) <- startProcess x (readModelName (unpack text))
                pure (Just consumer, x')
            | otherwise = putStrLn ("I: " ++ unpack text) >> pure jc

        go jc@(Just consumer, x) text
            | modelend `isPrefixOf` text = consumer Nothing >> pure (Nothing, x)
            | otherwise = consumer (Just text) >> pure jc


        readModelName ('M':'O':'D':'E':'L':rest) = case trimRight (trimLeft rest) of
            [] -> "unknown"
            name -> name

        readModelName name = trimRight (trimLeft name)


skipModule :: Maybe Text -> IO ()
skipModule _ = pure ()


devidePdbqtModulesFile :: FilePath -> FnProgressBar -> FnModelToFilename -> IO Int
devidePdbqtModulesFile inFilename progressBar moduleFileName = readPdbqtModels inFilename progressBar 0
    $ \ n name -> case moduleFileName name of
      Nothing -> pure (n, skipModule)
      (Just filename) -> do
        h <- openBinaryFile filename WriteMode
        let consumer = withJust (\text -> hPutStrLn h text) (hClose h)
        seq n (pure (n + 1, consumer))


listModels :: FilePath -> FnProgressBar -> IO [Text]
listModels inFilename progressBar = do
    names <- readPdbqtModels inFilename progressBar [] $ \ names name
      -> let n = pack name in seq n (pure (n : names, skipModule))
    pure $ reverse names
